using System;
using Server.Network;
using Server.Items;

namespace Server.Items
{
	public abstract class LockableContainer : TrapableContainer, ILockable, ILockpickable
	{
		private bool m_Locked;
		private int m_LockLevel;

		[CommandProperty( AccessLevel.GameMaster )]
		public int LockLevel
		{
			get
			{
				return m_LockLevel;
			}
			set
			{
				m_LockLevel = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public bool Locked
		{
			get
			{
				return m_Locked;
			}
			set
			{
				m_Locked = value;
			}
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 1 ); // version

			writer.Write( (int) m_LockLevel );
			writer.Write( (bool) m_Locked );

		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 1:
				{
					m_LockLevel = reader.ReadInt();

					goto case 0;
				}

				case 0:
				{
					m_Locked = reader.ReadBool();

					break;
				}
			}
		}

		public LockableContainer( int itemID, int gumpID, int defaultDropSound, Rectangle2D bounds ) : base( itemID, gumpID, defaultDropSound, bounds )
		{
			Key key = new Key( KeyType.Copper );

			key.Location = new Point3D( 100, 100, 0 );

			key.Locks.Add( this );

			AddItem( key );
		}

		public LockableContainer( Serial serial ) : base( serial )
		{
		}

		public override void OnDoubleClick( Mobile from )
		{
			if ( m_Locked )
			{
				int number;

				if ( from.AccessLevel >= AccessLevel.GameMaster )
				{
					number = 502502; // That is locked, but you open it with your godly powers.
					base.OnDoubleClick( from );
				}
				else
				{
					number = 502503; // That is locked.
				}

				from.Send( new MessageLocalized( Serial, ItemID, MessageType.Regular, 0x3B2, 3, number, "", "" ) );
			}
			else
			{
				base.OnDoubleClick( from );
			}
		}


		public void LockPick( Mobile from )
		{
			Locked = false;
		}
	}
}