using System;
using Server;

namespace Server.Misc
{
	public class SkillCheck
	{
		public static void Initialize()
		{
			Mobile.SkillCheckHandler = new SkillCheckHandler( Mobile_SkillCheck );
			Mobile.SkillCheckDirectHandler = new SkillCheckDirectHandler( Mobile_SkillCheckDirect );
		}

		public static bool Mobile_SkillCheck( Mobile from, SkillName skillName, double minSkill, double maxSkill )
		{
			Skill skill = from.Skills[skillName];

			if ( skill == null )
				return false;

			double value = skill.Value;

			if ( value < minSkill )
				return false; // Too difficult
			else if ( value >= maxSkill )
				return true; // No challenge

			double chance = (value - minSkill) / (maxSkill - minSkill);

			return CheckSkill( from, skill, chance );
		}

		public static bool Mobile_SkillCheckDirect( Mobile from, SkillName skillName, double chance )
		{
			Skill skill = from.Skills[skillName];

			if ( skill == null )
				return false;

			if ( chance < 0.0 )
				return false; // Too difficult
			else if ( chance >= 1.0 )
				return true; // No challenge

			return CheckSkill( from, skill, chance );
		}

		public static bool CheckSkill( Mobile from, Skill skill, double chance )
		{
			bool success = ( chance >= Utility.RandomDouble() );
			bool gain;

			if ( !success )
				gain = ( ((1.0 - chance) * 0.5) >= Utility.RandomDouble() );
			else
				gain = ( ((1.0 - chance) * 0.1) >= Utility.RandomDouble() );

			if ( gain )
				Gain( from, skill );

			return success;
		}

		public static void Gain( Mobile from, Skill skill )
		{
			if ( skill.Base < skill.Cap && skill.Lock == SkillLock.Up )
			{
				Skills skills = from.Skills;

				if ( skills.Total >= 7000 )
				{
					for ( int i = 0; i < skills.Length; ++i )
					{
						Skill toLower = skills[i];

						if ( toLower != skill && toLower.Lock == SkillLock.Down && toLower.BaseFixedPoint > 0 )
						{
							--toLower.BaseFixedPoint;
							break;
						}
					}
				}

				if ( skills.Total < 7000 )
				{
					++skill.BaseFixedPoint;

					StatLockType strLock = from.StrLock, dexLock = from.DexLock, intLock = from.IntLock;
					StatLockType up = StatLockType.Up, down = StatLockType.Down;

					SkillInfo info = skill.Info;

					int rawStr = from.RawStr, rawDex = from.RawDex, rawInt = from.RawInt;
					int rawTotal = rawStr + rawDex + rawInt;
					int rawCap = from.StatCap;

					if ( strLock == up && rawStr < 125 && ((info.StrGain / 100.0) >= Utility.RandomDouble()) )
					{
						if ( rawTotal >= rawCap )
						{
							if ( dexLock == down && rawDex > 10 && rawDex < rawInt )
							{
								--from.RawDex;
								++from.RawStr;
							}
							else if ( intLock == down && rawInt > 10 )
							{
								--from.RawInt;
								++from.RawStr;
							}
						}
						else
						{
							++from.RawStr;
						}
					}
					else if ( dexLock == up && rawDex < 125 && ((info.DexGain / 100.0) >= Utility.RandomDouble()) )
					{
						if ( rawTotal >= rawCap )
						{
							if ( strLock == down && rawStr > 10 && rawStr < rawInt )
							{
								--from.RawStr;
								++from.RawDex;
							}
							else if ( intLock == down && rawInt > 10 )
							{
								--from.RawInt;
								++from.RawStr;
							}
						}
						else
						{
							++from.RawDex;
						}
					}
					else if ( intLock == up && rawInt < 125 && ((info.IntGain / 100.0) >= Utility.RandomDouble()) )
					{
						if ( rawTotal >= rawCap )
						{
							if ( strLock == down && rawStr > 10 && rawStr < rawDex )
							{
								--from.RawStr;
								++from.RawInt;
							}
							else if ( dexLock == down && rawDex > 10 )
							{
								--from.RawDex;
								++from.RawInt;
							}
						}
						else
						{
							++from.RawInt;
						}
					}
				}
			}
		}
	}
}