using System;
using System.Collections;
using Server.Targeting;
using Server.Network;

//
// This is a first simple AI
//
//

namespace Server.Mobiles
{
	public class MeeleeAI : BaseAI
	{
		private enum MeeleeState
		{
            MeeleeWander,	// Act normaly
            MeeleeCombat,	// Have a combatant
			MeeleeGuard,	// Just lost a combatant so is in guard
			MeeleeFlee,		// Try to flee
			MeeleePanic		// Panic...stupid idea
		}

		private MeeleeState m_iState;
		private DateTime m_NextStopGuard;

		public MeeleeAI(BaseCreature m) : base (m)
		{
			m_iState = MeeleeState.MeeleeWander;
		}

		public override bool Think()
		{
			if ( m_Mobile.Deleted )
				return false;

			switch (m_iState)
			{
				case MeeleeState.MeeleeWander:
					return DoWander();

				case MeeleeState.MeeleeCombat:
					return DoCombat();

				case MeeleeState.MeeleeGuard:
					return DoGuard();

				case MeeleeState.MeeleeFlee:
					return DoFlee();

				case MeeleeState.MeeleePanic:
					return DoPanic();

				default:
					return false;
			}
		}

		public virtual bool DoWander()
		{
			m_Mobile.DebugSay("I have no combatant");

			if (AquireFocusMob(m_Mobile.RangePerception, AcquireType.Closest, false, false, true))
			{
				m_Mobile.DebugSay("I have detected " + m_Mobile.FocusMob.Name + " and I will attack");
				m_Mobile.Combatant = m_Mobile.FocusMob;
				m_Mobile.FocusMob = null;
				m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
				m_iState = MeeleeState.MeeleeCombat;
			}
			else
			{
				DoWalkRandomSlowly(5, 1);
			}

			return true;
		}

		public virtual bool DoCombat()
		{
			if ( m_Mobile.Combatant == null || m_Mobile.Combatant.Deleted )
			{
				m_Mobile.DebugSay("My combatant is deleted");
				m_iState = MeeleeState.MeeleeGuard;
				m_NextStopGuard = DateTime.Now + TimeSpan.FromSeconds( 10 );
				return true;
			}

			if (DoWalkMobileRange(m_Mobile.Combatant, 1, true, m_Mobile.RangeFight, m_Mobile.RangeFight))
			{
				// Be sure to face the combatant
				m_Mobile.Direction = m_Mobile.GetDirectionTo(m_Mobile.Combatant.Location);
			}
			else
			{
				if ( m_Mobile.Combatant != null )
				{
					m_Mobile.DebugSay("I am still not in range of " + m_Mobile.Combatant.Name);

					if ( (int) m_Mobile.GetDistanceToSqrt( m_Mobile.Combatant ) > m_Mobile.RangePerception + 1 )
					{
						m_Mobile.DebugSay("I have lost " + m_Mobile.Combatant.Name);
						m_Mobile.Combatant = null;
						m_iState = MeeleeState.MeeleeGuard;
						m_NextStopGuard = DateTime.Now + TimeSpan.FromSeconds( 10 );
						return true;
					}
				}
			}

			// At 20% we should check if we must leave
			if ( m_Mobile.Hits < m_Mobile.HitsMax*20/100 )
			{
				bool bFlee = false;
				// if my current hits are more than my opponent, i don't care
				if (m_Mobile.Hits < m_Mobile.Combatant.Hits)
				{
					int iDiff = m_Mobile.Combatant.Hits - m_Mobile.Hits;

					if ( Utility.Random(0, 100) > 10 + iDiff) // 10% to flee + the diff of hits
					{
						bFlee = true;
					}


				}
				else if (m_Mobile.Hits >= m_Mobile.Combatant.Hits)
				{
					if ( Utility.Random(0, 100) > 10 ) // 10% to flee
					{
						bFlee = true;
					}
				}
						
				if (bFlee)
				{
					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					m_iState = MeeleeState.MeeleeFlee; 
				}
			}

			// At 10% we panic
			if ( m_Mobile.Hits < m_Mobile.HitsMax*10/100 )
			{
				if ( Utility.Random(0, 100) > 10 ) // 10% to panic
				{
					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					m_iState = MeeleeState.MeeleePanic; 
				}
			}

			return true;
		}

		public virtual bool DoGuard()
		{
			if ( DateTime.Now < m_NextStopGuard )
			{
				m_Mobile.DebugSay("I am on guard");

				m_Mobile.Turn( Utility.Random(0, 2) - 1 );

				if (AquireFocusMob(m_Mobile.RangePerception, AcquireType.Closest, false, false, true))
				{
					m_Mobile.DebugSay("I have detected " + m_Mobile.FocusMob.Name + " and I will attack");
					m_Mobile.Combatant = m_Mobile.FocusMob;
					m_Mobile.FocusMob = null;
					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					m_iState = MeeleeState.MeeleeCombat;
				}

			}
			else
			{
				m_Mobile.DebugSay("I stop be in Guard");

				m_Mobile.FocusMob = null;
				m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
				m_iState = MeeleeState.MeeleeWander;
			}

			return true;
		}

		public virtual bool DoFlee()
		{
			if ( m_Mobile.Combatant == null || m_Mobile.Combatant.Deleted )
			{
				m_Mobile.DebugSay("My combatant is deleted");
				m_iState = MeeleeState.MeeleeGuard;
				m_NextStopGuard = DateTime.Now + TimeSpan.FromSeconds( 10 );
				return true;
			}

			if (DoWalkMobileRange(m_Mobile.Combatant, 3, true, m_Mobile.RangePerception*2, m_Mobile.RangePerception*3))
			{
				m_Mobile.DebugSay("I Have fled");
				m_iState = MeeleeState.MeeleeGuard;
				m_NextStopGuard = DateTime.Now + TimeSpan.FromSeconds( 10 );
				return true;                        
			}
			else
			{
				m_Mobile.DebugSay("I am fleeing!");
			}

			return true;
		}

		public virtual bool DoPanic()
		{
			m_Mobile.DebugSay("*panic*");
			DoWalkRandomNonStop(2, 4);

			return true;
		}
	}
}

// Archive : Find another fighter
/*
if (AquireFocusMobClosest(m_Mobile.RangePerception))
{
	if (m_Mobile.FocusMob != m_Mobile.Combatant)
	{
		if (m_Mobile.FocusMob != null)
		{
			m_Mobile.DebugSay(m_Mobile.FocusMob.Name + " is closer than " + m_Mobile.Combatant.Name);

			if (Utility.Random(0, 10) >= 5)
			{
				m_Mobile.DebugSay("I have keeped my combatant " + m_Mobile.Combatant.Name);
			}
			else
			{
				m_Mobile.Combatant = m_Mobile.FocusMob;
				m_Mobile.DebugSay("I have changed my combatant to" + m_Mobile.Combatant.Name);
			}
		}
	}
}
*/