using System;
using System.Collections;
using Server.Misc;
using Server.Spells;
using Server.Spells.First;
using Server.Spells.Second;
using Server.Spells.Third;
using Server.Spells.Fourth;
using Server.Spells.Fifth;
using Server.Spells.Sixth;
using Server.Spells.Seventh;
using Server.Targeting;

namespace Server.Mobiles
{
	[Dispellable]
	public class EnergyVortex : BaseAggressive
	{
		private Timer m_Timer;

		[Constructable]
		public EnergyVortex()
		{
			this.Body = 164; // (Utility.Random( 1000 ) == 0) ? 0xDC : 164; // 1 in 1000 chance to be a purple llama
			//this.Hue = 0x486; // TODO: Proper purple color
			this.Kills = 500; // Murderer
			this.Name = "an energy vortex";
			this.BaseSoundID = 0x107;

			this.InitStats( 500, 450, 500 );

			this.Skills[SkillName.Wrestling].Base = 100.0;
			this.Skills[SkillName.Tactics].Base = 100.0;
			this.Skills[SkillName.Anatomy].Base = 100.0;
			this.Skills[SkillName.MagicResist].Base = 100.0;
			this.Skills[SkillName.Magery].Base = 100.0;
			this.Skills[SkillName.EvalInt].Base = 100.0;
			this.Skills[SkillName.Meditation].Base = 100.0;

			m_Timer = new ExpireTimer( this );
			m_Timer.Start();
		}

		public EnergyVortex( Serial serial ) : base( serial )
		{
			m_Timer = new ExpireTimer( this );
			m_Timer.Start();
		}

		public override void OnAfterDelete()
		{
			m_Timer.Stop();

			base.OnAfterDelete();
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override double GetValueFrom( Mobile m )
		{
			return (m.Skills[SkillName.Magery].Value + m.Int) / this.GetDistanceToSqrt( m );
		}

		private class ExpireTimer : Timer
		{
			private EnergyVortex m_Owner;

			public ExpireTimer( EnergyVortex owner ) : base( TimeSpan.FromMinutes( 1.5 ) )
			{
				this.Priority = TimerPriority.FiveSeconds;

				m_Owner = owner;
			}

			protected override void OnTick()
			{
				m_Owner.Delete();
			}
		}
	}
}