using System;
using Server;
using Server.Mobiles;
using Server.Spells;
using Server.Spells.Seventh;
using Server.Spells.Fourth;
using Server.Spells.Sixth;

namespace Server.Regions
{
	public class FeluccaTown : Region
	{
		public static void Initialize()
		{
			Region.AddRegion( new FeluccaTown( "Cove", typeof( ArcherGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Britain", typeof( WarriorGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Britain Graveyard", typeof( WarriorGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Lord British's Castle", typeof( WarriorGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Lord Blackthorne's Castle", typeof( WarriorGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Jhelom", typeof( ArcherGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Minoc", typeof( ArcherGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Ocllo", typeof( WarriorGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Trinsic", typeof( WarriorGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Vesper", typeof( WarriorGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Yew", typeof( ArcherGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Wind", typeof( WarriorGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Serpent's Hold", typeof( ArcherGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Skara Brae", typeof( ArcherGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Nujel'm", typeof( WarriorGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Moonglow", typeof( WarriorGuard ) ) );
			Region.AddRegion( new FeluccaTown( "Magincia", typeof( WarriorGuard ) ) );
			Region.AddRegion( new Region( "the town of", "Buccaneer's Den", Map.Felucca ) );//not a town, no guards, use all default behavior
		}

		private Type m_GuardType;
		private static object[] m_GuardParams = new object[1];

		public FeluccaTown( string name, Type guardType ) : base( "the town of", name, Map.Felucca )
		{
			m_GuardType = guardType;
		}

		public override void MakeGuard( Mobile focus )
		{
			m_GuardParams[0] = focus;

			Activator.CreateInstance( m_GuardType, m_GuardParams );
		}

		public override void OnEnter( Mobile m )
		{
			base.OnEnter( m );
			m.SendMessage( "You are now under the protection of the guards." );

			if ( m.Kills >= 5 )
				BaseGuard.Spawn( m );
		}

		public override void OnExit( Mobile m )
		{
			base.OnExit( m );

			m.SendMessage( "You have left the protection of the guards." );
		}

		public override void OnSpeech( SpeechEventArgs args )
		{
			if ( args.HasKeyword( 0x0007 ) ) // *guards*
			{
				IPooledEnumerable eable = Map.GetMobilesInRange( args.Mobile.Location, 15 );

				foreach ( Mobile m in eable )
				{
					if ( ( m.Kills >= 5 || m.Criminal ) && m.Alive && !(m is BaseGuard) )
						BaseGuard.Spawn( m );
				}

				eable.Free();
			}
		}

		public override void OnCriminalAction( Mobile m )
		{
			if ( m.Alive && !(m is BaseGuard) )
				BaseGuard.Spawn( m );
		}
	}

	public class FeluccaDungeon : Region
	{
		public static void Initialize()
		{
			Region.AddRegion( new FeluccaDungeon( "Covetous" ) );
			Region.AddRegion( new FeluccaDungeon( "Deceit" ) );
			Region.AddRegion( new FeluccaDungeon( "Despise" ) );
			Region.AddRegion( new FeluccaDungeon( "Destard" ) );
			Region.AddRegion( new FeluccaDungeon( "Hythloth" ) );
			Region.AddRegion( new FeluccaDungeon( "Shame" ) );
			Region.AddRegion( new FeluccaDungeon( "Wrong" ) );
		}

		public FeluccaDungeon( string name ) : base( "the dungeon", name, Map.Felucca )
		{
		}

		public override bool OnBeginSpellCast( Mobile m, Spell s )
		{
			if ( s is GateTravelSpell || s is RecallSpell || s is MarkSpell )
			{
				m.SendMessage( "You cannot travel from here." );
				return false;
			} else
				return base.OnBeginSpellCast( m, s );
		}
	}							  
}

