using System;
using Server;
using Server.Mobiles;
using Server.Spells;
using Server.Spells.Seventh;
using Server.Spells.Fourth;
using Server.Spells.Sixth;

namespace Server.Regions
{
	public class TrammelTown : Region
	{
		public static void Initialize()
		{
			Region.AddRegion( new TrammelTown( "Cove", typeof( ArcherGuard ) ) );
			Region.AddRegion( new TrammelTown( "Britain", typeof( WarriorGuard ) ) );
			Region.AddRegion( new TrammelTown( "Britain Graveyard", typeof( WarriorGuard ) ) );
			Region.AddRegion( new TrammelTown( "Lord British's Castle", typeof( WarriorGuard ) ) );
			Region.AddRegion( new TrammelTown( "Lord Blackthorne's Castle", typeof( WarriorGuard ) ) );
			Region.AddRegion( new TrammelTown( "Jhelom", typeof( ArcherGuard ) ) );
			Region.AddRegion( new TrammelTown( "Minoc", typeof( ArcherGuard ) ) );
			Region.AddRegion( new TrammelTown( "Ocllo", typeof( WarriorGuard ) ) );
			Region.AddRegion( new TrammelTown( "Trinsic", typeof( WarriorGuard ) ) );
			Region.AddRegion( new TrammelTown( "Vesper", typeof( WarriorGuard ) ) );
			Region.AddRegion( new TrammelTown( "Yew", typeof( ArcherGuard ) ) );
			Region.AddRegion( new TrammelTown( "Wind", typeof( WarriorGuard ) ) );
			Region.AddRegion( new TrammelTown( "Serpent's Hold", typeof( ArcherGuard ) ) );
			Region.AddRegion( new TrammelTown( "Skara Brae", typeof( ArcherGuard ) ) );
			Region.AddRegion( new TrammelTown( "Nujel'm", typeof( WarriorGuard ) ) );
			Region.AddRegion( new TrammelTown( "Moonglow", typeof( WarriorGuard ) ) );
			Region.AddRegion( new TrammelTown( "Magincia", typeof( WarriorGuard ) ) );
		}

		private Type m_GuardType;
		private static object[] m_GuardParams = new object[1];

		public TrammelTown( string name, Type guardType ) : base( "the town of", name, Map.Trammel )
		{
			m_GuardType = guardType;
		}

		public override void MakeGuard( Mobile focus )
		{
			m_GuardParams[0] = focus;

			Activator.CreateInstance( m_GuardType, m_GuardParams );
		}

		public override void OnEnter( Mobile m )
		{
			base.OnEnter( m );
			m.SendMessage( "You are now under the protection of the guards." );

			if ( m.Kills >= 5 )
				BaseGuard.Spawn( m );
		}

		public override void OnExit( Mobile m )
		{
			base.OnExit( m );

			m.SendMessage( "You have left the protection of the guards." );
		}

		public override void OnSpeech( SpeechEventArgs args )
		{
			if ( args.HasKeyword( 0x0007 ) ) // *guards*
			{
				IPooledEnumerable eable = Map.GetMobilesInRange( args.Mobile.Location, 15 );

				foreach ( Mobile m in eable )
				{
					if ( ( m.Kills >= 5 || m.Criminal ) && m.Alive && !(m is BaseGuard) )
						BaseGuard.Spawn( m );
				}

				eable.Free();
			}
		}

		public override bool AllowHarmful( Mobile from, Mobile target )
		{
			int notor = Notoriety.Compute( from, target );
			if ( notor == Notoriety.Innocent && !(from is BaseGuard) )
				return false;
			else
				return true;
		}

		public override bool AllowBenificial( Mobile from, Mobile target )
		{
			int notor = Notoriety.Compute( from, target );
			if ( notor == Notoriety.Criminal || notor == Notoriety.Murderer )
				return false;
			else
				return true;
		}

		public override void OnCriminalAction( Mobile m )
		{
			if ( m.Alive && !(m is BaseGuard) )
				BaseGuard.Spawn( m );
		}
	}

	public class TrammelDungeon : Region
	{
		public static void Initialize()
		{
			Region.AddRegion( new TrammelDungeon( "Covetous" ) );
			Region.AddRegion( new TrammelDungeon( "Deceit" ) );
			Region.AddRegion( new TrammelDungeon( "Despise" ) );
			Region.AddRegion( new TrammelDungeon( "Destard" ) );
			Region.AddRegion( new TrammelDungeon( "Hythloth" ) );
			Region.AddRegion( new TrammelDungeon( "Shame" ) );
			Region.AddRegion( new TrammelDungeon( "Wrong" ) );
		}

		public TrammelDungeon( string name ) : base( "the dungeon", name, Map.Felucca )
		{
		}

		public override bool OnBeginSpellCast( Mobile m, Spell s )
		{
			if ( s is GateTravelSpell || s is RecallSpell || s is MarkSpell )
			{
				m.SendMessage( "You cannot travel from here." );
				return false;
			} 
			else
				return base.OnBeginSpellCast( m, s );
		}

		public override bool AllowHarmful( Mobile from, Mobile target )
		{
			int notor = Notoriety.Compute( from, target );
			if ( notor == Notoriety.Innocent && !(from is BaseGuard) )
				return false;
			else
				return true;
		}

		public override bool AllowBenificial( Mobile from, Mobile target )
		{
			int notor = Notoriety.Compute( from, target );
			if ( notor == Notoriety.Criminal || notor == Notoriety.Murderer )
				return false;
			else
				return true;
		}
	}	
}
