using System;
using Server.Targeting;
using Server.Network;

namespace Server.Spells.Sixth
{
	public class EnergyBoltSpell : Spell
	{
		private static SpellInfo m_Info = new SpellInfo(
				"Energy Bolt", "Corp Por",
				SpellCircle.Sixth,
				230,
				Reagent.BlackPearl,
				Reagent.Nightshade
			);

		public EnergyBoltSpell( Mobile caster, Server.Items.SpellScroll scroll ) : base( caster, scroll, m_Info )
		{
		}

		public override void OnCast()
		{
			Caster.Target = new InternalTarget( this );
		}

		public void Target( Mobile m )
		{
			if ( !Caster.CanSee( m ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( CheckHSequence( m ) )
			{
				// Base damage 20-50
				double damage = Utility.Random( 20, 30 );

				if ( CheckResisted( m ) )
				{	// Resisted, take half damage
					damage *= 0.5;

					m.SendLocalizedMessage( 501783 ); // You feel yourself resisting magical energy.
				}

				// Scale damage based on evalint and resist
				damage *= GetDamageScalar( m );

				if ( Caster != m )
					Caster.Direction = Caster.GetDirectionTo( m ); // Turn to them

				// Do the effects
				Caster.PlaySound( 0x20A );
				Effects.SendMovingEffect( Caster, m, 0x379F, 7, 0, false, true );

				// Deal the damage
				m.Damage( (int) damage );
			}

			FinishSequence();
		}

		private class InternalTarget : Target
		{
			private EnergyBoltSpell m_Owner;

			public InternalTarget( EnergyBoltSpell owner ) : base( 12, false, TargetFlags.Harmful )
			{
				m_Owner = owner;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				if ( o is Mobile )
				{
					m_Owner.Target( (Mobile)o );
				}
			}

			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}